@@ -780,9 +780,9 @@ static int ulfius_webservice_dispatcher (void * cls,
                                            o_strcasestr(u_map_get_case(con_info->request->map_header, "upgrade"), U_WEBSOCKET_UPGRADE_VALUE)==NULL?"No Upgrade websocket header\n":"",
                                            o_strcasestr(u_map_get_case(con_info->request->map_header, "Connection"), "Upgrade")==NULL?"No Connection Upgrade header\n":"",
                                            u_map_get_case(con_info->request->map_header, "Sec-WebSocket-Key")==NULL?"No Sec-WebSocket-Key header\n":"",
-                                           o_strcmp(con_info->request->http_protocol, "HTTP/1.1")!=0?"Wrong HTTP Protocol":"",
+                                           o_strcmp(con_info->request->http_protocol, "HTTP/1.1")!=0?"Wrong HTTP Protocol\n":"",
                                            o_strcmp(u_map_get_case(con_info->request->map_header, "Sec-WebSocket-Version"), "13")!=0?"Wrong websocket version\n":"",
-                                           o_strcmp(con_info->request->http_verb, "GET")!=0?"Method is not GET":"");
+                                           o_strcmp(con_info->request->http_verb, "GET")!=0?"Method is not GET\n":"");
                 response->status = MHD_HTTP_BAD_REQUEST;
                 y_log_message(Y_LOG_LEVEL_DEBUG, "Ulfius - Error websocket connection: %s", response_buffer);
                 mhd_response = MHD_CREATE_RESPONSE_FROM_BUFFER_PIMPED (o_strlen(response_buffer), response_buffer, mhd_response_flag );
@@ -999,6 +999,9 @@ static int ulfius_webservice_dispatcher (void * cls,
           }
           MHD_destroy_response (mhd_response);
           // Free Response parameters
+          if (response->free_shared_data != NULL && response->shared_data != NULL) {
+            response->free_shared_data(response->shared_data);
+          }
           ulfius_clean_response_full(response);
           response = NULL;
         }
